<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 2005 Ernesto Baschny (ernst@cron-it.de)
*  (c) 2004 Jan Wischnat, the <dfn> routine
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is 
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/** 
 * IProcFunc for 'cron_accessiblemenus' extension.
 *
 * @author	Ernesto Baschny <ernst@cron-it.de>
 * @coauthor    Jan Wischnat, schwangau.de
 */

class user_cronaccessiblemenus {

	/**
	 * Adds accesskeys to a menu
	 *
	 * @param	array		The $menuArr array which simply is a num-array of page records which goes into the menu.
	 * @param	array		TypoScript configuration for the function. Notice that the property "parentObj" is a reference to the parent (calling) object (the tslib_Xmenu class instantiated)
	 * @return	array		The modified $menuArr array
	 */
	function addAccessKeys($menuArr, $conf) {
		if (!$GLOBALS['TSFE']->register['usedAccessKeys']) {
			$GLOBALS['TSFE']->register['usedAccessKeys'] = array();
		}
		$forbiddenKeys = array();
		if ($conf['forbiddenKeys']) {
			$forbiddenKeys = t3lib_div::trimExplode(',', strtoupper($conf['forbiddenKeys']), 1);
		}
		if ($menuArr['val']['doNotLinkIt']) { return $menuArr; }
		$hasKey = FALSE;
		$key = '';

		$tx_govaccesskey_accesskey = $conf['parentObj']->menuArr[$menuArr['key']]['tx_govaccesskey_accesskey'];
		if ($tx_govaccesskey_accesskey != '') {
			// First use the tx_govaccesskey_accesskey, if set
			$key = $tx_govaccesskey_accesskey;
			$hasKey = TRUE;
		}
		if (!$hasKey) {
			// Check if we have already set an accesskey for this target
			foreach ($GLOBALS['TSFE']->register['usedAccessKeys'] as $k => $h) {
				if ($menuArr['linkHREF'] == $h) {
					// We already had this target on this page, use the same accesskey
					$hasKey = TRUE;
					$key = $k;
				}
			}
		}
		if (!$hasKey && !$conf['accessKeys.']['noAutoGenerated']) {
			// Fallback, autogenerate the key from the menu title
			$title = $GLOBALS['TSFE']->csConvObj->specCharsToASCII($GLOBALS['TSFE']->renderCharset, $menuArr['parts']['title']);
			for ($i = 0; $i < strlen($title); $i++) {
				$key = strtoupper(substr($title, $i, 1));
				if (!preg_match('/^[A-Z0-9]$/', $key)) { continue; }
				if (!isset($GLOBALS['TSFE']->register['usedAccessKeys'][$key]) && !in_array($key, $forbiddenKeys)) {
					$GLOBALS['TSFE']->register['usedAccessKeys'][$key] = $menuArr['linkHREF'];
					$hasKey = TRUE;
					break;
				}
			}
		}
		if (preg_match('/accesskey="?([^"])"?/', $menuArr['parts']['ATag_begin'], $matches)) {
			// Another accesskey already set for whatever reason, leave it alone
			$key = strtoupper($matches[1]);
			$hasKey = TRUE;
			$GLOBALS['TSFE']->register['usedAccessKeys'][$key] = $menuArr['linkHREF'];
		} else if ($key != '' && $hasKey) {
			// Add the accesskey property to the <a> tag
			if (preg_match('/accesskey=""/', $menuArr['parts']['ATag_begin'])) {
				// Empty accesskey found
				$menuArr['parts']['ATag_begin'] = preg_replace('/accesskey=""/', 'accesskey="' . $key . '"', $menuArr['parts']['ATag_begin']);
			} else {
				// No accesskey yet, add the new one
				$menuArr['parts']['ATag_begin'] = preg_replace('/>$/', ' accesskey="' . $key . '" >', $menuArr['parts']['ATag_begin']);
			}
		}
		// Wrap the key in the title, if wanted
		if ($key != '' && $hasKey && $conf['accessKeyWrap']) {
			$menuArr['parts']['title'] = preg_replace('/(' . $key . ')/i', $this->cObj->wrap('$1', $conf['accessKeyWrap']), $menuArr['parts']['title'], 1);
		}
		// Append something to the title, if wanted
		if ($key != '' && $hasKey && $conf['appendWrap']) {
			$menuArr['parts']['title'] = $menuArr['parts']['title'] . $this->cObj->wrap($key, $conf['appendWrap']);
		}
		return $menuArr;
	}

	/**
	 * Adds dfn to a menu
	 * by Jan Wischnat
	 *
	 * @param	array		The $menuArr array which simply is a num-array of page records which goes into the menu.
	 * @param	array		TypoScript configuration for the function. Notice that the property "parentObj" is a reference to the parent (calling) object (the tslib_Xmenu class instantiated)
	 * @return	array		The modified $menuArr array
	 */
	function addDfn($menuArr, $conf) {
		$itemRow = $conf['parentObj']->menuArr[$menuArr['key']];
		$level = $conf['parentObj']->menuNumber;

		$key = $menuArr['key'];
		if ($conf['dfn.']['start']) {
			$key = $menuArr['key'] + $conf['dfn.']['start'] - 1;
		}
		if ($conf['dfn.']['menuSet']) {
			$menuSet = 'dfn_lev' . $level . '_' . $conf['dfn.']['menuSet'];
			if ($GLOBALS['TSFE']->register[$menuSet]) {
				$key = $GLOBALS['TSFE']->register[$menuSet];
			}
		}
		switch ($level) {
		case 1:
			$dfn = ($key+1);
			$GLOBALS['TSFE']->register['dfn_lev1'] = $dfn;
			break;

		case 2:
			$dfn = $GLOBALS['TSFE']->register['dfn_lev1'].'.'.($key+1);
			$GLOBALS['TSFE']->register['dfn_lev2'] = $dfn;
			break;

		case 3:
			$dfn = $GLOBALS['TSFE']->register['dfn_lev2'].'.'.($key+1);
			$GLOBALS['TSFE']->register['dfn_lev3'] = $dfn;
			break;

		case 4:
			$dfn = $GLOBALS['TSFE']->register['dfn_lev3'].'.'.($key+1);
			break;

		default:
			$dfn = '';
			break;
		}
		if ($dfn) {
			if ($conf['dfn.']['menuSet']) {
				$GLOBALS['TSFE']->register[$menuSet] = $key+1;
			}
			if ($conf['dfn.']['stdWrap.']) {
				$dfn = $this->cObj->stdWrap($dfn, $conf['dfn.']['stdWrap.']);
			}
			if ($conf['dfn.']['inATag'])	{
				$menuArr['parts']['title'] = '<dfn>'.$dfn.'</dfn>' . $menuArr['parts']['title'];
			} else	{
				$menuArr['parts']['before'] .= '<dfn>'.$dfn.'</dfn>';
			}
		}
		return $menuArr;
	}

	/**
	 * Makes a TMENU accessible
	 *
	 * @param	array		The $menuArr array which simply is a num-array of page records which goes into the menu.
	 * @param	array		TypoScript configuration for the function. Notice that the property "parentObj" is a reference to the parent (calling) object (the tslib_Xmenu class instantiated)
	 * @return	array		The modified $menuArr array
	 */
	function makeAccessible($menuArr, $conf) {
		if ($conf['accessKeys']) {
			$menuArr = $this->addAccessKeys($menuArr, $conf);
		}
		if ($conf['dfn']) {
			$menuArr = $this->addDfn($menuArr, $conf);
		}
		$this->menuArr = $menuArr;
		$this->conf = $conf;
		// Call hook for other possible accessibility features
		if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['ext/cron_accessiblemenus/class.user_cronaccessiblemenus.php']['makeAccessible'])) {
			$_params = array('pObj' => &$this);
			foreach($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['ext/cron_accessiblemenus/class.user_cronaccessiblemenus.php']['makeAccessible'] as $_funcRef) {
				$menuArr = t3lib_div::callUserFunction($_funcRef,$_params,$this);
			}
		}
		return $menuArr;
	}

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cron_accessiblemenus/class.user_cronaccessiblemenus.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cron_accessiblemenus/class.user_cronaccessiblemenus.php']);
}

?>